'use strict';

/**
* A promise object provided by the q promise library.
* @external Promise
* @see {@link https://docs.angularjs.org/api/ng/service/$q}
**/

/**
 * @ngdoc overview
 * @name app
 * @description
 * ## Module: app
 * Mòdul únic de l'aplicació, gestiona el login i registre dels usuaris, cerques, llistats de l'aplicació  comAPPcasa
 */
var app = angular.module('comAPPcasa', ['ionic','firebase','pascalprecht.translate', 'ngCordova', 'ionic.contrib.drawer.vertical'])

/*
------------------------------------
Configuració de l'aplicació
------------------------------------
*/
app.config(function($ionicConfigProvider, $stateProvider, $urlRouterProvider, $translateProvider) {


  //Per configurar l'aspecte dels checkbox
  $ionicConfigProvider.form.checkbox("circle");

  $stateProvider
    .state('app', {
      url: "/app",
      abstract: true,
      cache: false,
      controller : 'MainCtrl',
      templateUrl: "templates/menu.html"
    })
    // Portada
    .state('app.home', {
      url: '/home',
      views: {
        'appContent' :{
        controller : 'HomeCtrl',
        templateUrl: 'templates/home.html'
        }
      }
    })
    // Pantalla de login
    .state('app.login', {
    url: '/login',
      views: {
        'appContent' :{
          cache: false,
          controller : 'LoginCtrl',
          templateUrl: 'templates/login.html'
        }
      }
    })
    // Vista de creació de comptes  d'usuari
    .state('app.signup', {
    url: '/signup',
      views: {
        'appContent' :{
          cache: false,
          controller : 'SignUpCtrl',
          templateUrl: 'templates/signup.html'
        }
      }
    })
    // Vista de recuperació de la contrasenya d'usuari
    .state('app.lostpassword', {
    url: '/lostpassword',
      views: {
        'appContent' :{
          cache: false,
          controller : 'LostPasswordCtrl',
          templateUrl: 'templates/lostpassword.html'
        }
      }
    })

    //Vista dels resultats de cerca. Conté les pestanyes Llistat / Mapa i Comparativa
    .state('app.cerca', {
      url: '/cerca',
      abstract: true,
      cache: false,
      views: {
        'appContent' :{
          controller : 'CercaCtrl',
          templateUrl: 'templates/cerca.html',
        }
      }
    })
        //Pestanya llistat de fotos dels resultats de cerca
        .state('app.cerca.llistat', {
          url: '/llistat',
          cache: false,
          views: {
            'llistat-tab' :{
              //Utilitza el controlador del parent
              templateUrl: 'templates/cerca.llistat.html',
            }
          }
        })
        //Pestanta mapa amb els marcadors dels resultats de cerca
        .state('app.cerca.mapa', {
          url: '/mapa',
          cache: false,
          views: {
            'mapa-tab' :{
              controller : 'CercaMapaCtrl',
              templateUrl: 'templates/cerca.mapa.html',
            }
          }
        })
        .state('app.cerca.posicio', {
          url: '/mapa/:posicio',
          views: {
            'mapa-tab' :{
              controller : 'CercaMapaCtrl',
              templateUrl: 'templates/cerca.mapa.html',
            }
          }
        })
        //Pestanya comparativa als resultats de cerca
        .state('app.cerca.comparativa', {
          url: '/comparativa',
          cache: false,
          views: {
            'comparativa-tab' :{
              controller : 'ComparativaCtrl',
              templateUrl: 'templates/cerca.comparativa.html',
            }
          }
        })
    // Vista de fitxa d'un habitatge
    .state('app.fitxa', {
      url: '/fitxa/:key',
      views: {
        'appContent' :{
          controller : 'FitxaCtrl',
          templateUrl: 'templates/fitxa.html',
        }
      }
    })
    // Vista del mapa d'un habitatge
    .state('app.fitxamapa', {
      url: '/fitxamapa/:key',
      views: {
        'appContent' :{
          controller : 'FitxaMapaCtrl',
          templateUrl: 'templates/fitxa_mapa.html',
        }
      }
    })
    // Vista de creació d'anuncis
    .state('app.nouanunci', {
      url: '/nouanunci',
      loginRequired: true,
      cache: false,
      views: {
        'appContent' :{
          controller : 'FormAnunciCtrl',
          templateUrl: 'templates/form-anunci.html',
        }
      }
    })
    // Vista de modificació d'anuncis
    .state('app.modianunci', {
      url: '/modianunci/:key',
      loginRequired: true,
      cache: false,
      views: {
        'appContent' :{
          controller : 'FormAnunciCtrl',
          templateUrl: 'templates/form-anunci.html',
        }
      }
    })
    // Vista de llistat d'anuncis de l'usuari venedor
    .state('app.anuncis', {
    url: '/anuncis',
    loginRequired: true,
    cache: false,
      views: {
        'appContent' :{
          controller : 'LlistaAnuncisCtrl',
          templateUrl: 'templates/anuncis.html',
        }
      }
    })
    // Vista de llistat d'anuncis favorits de l'usuari
    .state('app.favorits', {
    url: '/favorits',
    loginRequired: true,
    cache: false,
      views: {
        'appContent' :{
          controller : 'FavoritsCtrl',
          templateUrl: 'templates/favorits.html',
        }
      }
    })
    // Vista del formulari de modificació del compte
    .state('app.perfil', {
    url: '/perfil',
    loginRequired: true,
    cache: false,
      views: {
        'appContent' :{
          controller : 'PerfilCtrl',
          templateUrl: 'templates/perfil.html',
        }
      }
    })
    // Vista del  la pantalla de configuració de l'apliacació
    .state('app.config', {
    url: '/config',
    loginRequired: false,
    cache: false,
      views: {
        'appContent' :{
          controller : 'ConfigCtrl',
          templateUrl: 'templates/config.html',
        }
      }
    })

  // La vista a mostrar per defecte: Pantalla de benvinguda
  $urlRouterProvider.otherwise('/app/home')

  //Carreguem els fitxers de traducció i definim els paràmetres per defecte
  for(var lang in translations){
    $translateProvider.translations(lang, translations[lang]);
  }
  //Idioma per defecte
  $translateProvider.preferredLanguage('ca');
  //Definim l'estratègia per defecte a les traduccions
  $translateProvider.useSanitizeValueStrategy('escape');

});


/*
------------------------------------
Run
------------------------------------
*/
app.run(function($ionicPlatform, $rootScope, $state, Auth) {
  $ionicPlatform.ready(function() {
    if(window.cordova && window.cordova.plugins.Keyboard) {
      cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
      cordova.plugins.Keyboard.disableScroll(true);
    }
    if(window.StatusBar) {
      StatusBar.styleDefault();
    }

    $rootScope.postLogInState = null;

    //Definició de variables estàtiques, disponibles a tota l'aplicació
    $rootScope.tipusAnunci = {
      2: 'lang_estat_anunci_inactiu',
      1: 'lang_estat_anunci_actiu'
    };

    $rootScope.tipusOpcions = {
      venta: 'lang_venta',
      lloguer: 'lang_lloguer'
    };

    $rootScope.estatsHabitatge = {
      99: 'lang_estats_qualsevol',
      1: 'lang_estats_nou',
      2: 'lang_estats_seminou',
      6: 'lang_estats_segonama',
      3: 'lang_estats_reformat',
      4: 'lang_estats_per_reformar',
      5: 'lang_estats_ruina'
    };

    $rootScope.ordenacions = {
      'preu_asc': {text: 'lang_order_preu_asc', camp:'preu', ordre:'asc'},
      'preu_desc': {text:'lang_order_preu_desc', camp:'preu', ordre:'desc'},
      'superficie_asc': {text:'lang_order_superficie_asc', camp:'superficie', ordre:'asc'},
      'superficie_desc': {text:'lang_order_superficie_desc', camp:'superficie', ordre:'desc'},
      'creation_asc': {text:'lang_order_creation_asc', camp:'creation', ordre:'asc'},
      'creation_desc': {text:'lang_order_creation_desc', camp:'creation', ordre:'desc'},
      'numhab_asc': {text:'lang_order_hab_asc', camp:'numhab', ordre:'asc'},
      'numhab_desc': {text:'lang_order_hab_desc', camp:'numhab', ordre:'desc'},
      'numlavabos_asc': {text:'lang_order_lavabos_asc', camp:'numlavabos', ordre:'asc'},
      'numlavabos_desc': {text:'lang_order_lavabos_desc', camp:'numlavabos', ordre:'desc'},
    }

    $rootScope.idiomes = {
      'ca': 'lang_idioma_catala',
      'es': 'lang_idioma_castellano',
      'en': 'lang_idioma_english',
    };

    /**
     * ngdoc event
     * name .#stateChangeStart //the name after the hash is important
     * eventOf . // the same part as above other than the hash
     * eventType  on  // **`on`** is a keyword you can choose to either emit or broadcast on the target
     */
    $rootScope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParamss){
      // Si la pantalla és privada i l'usuari
      //no està identificat, mostrem al login.
      if (toState.loginRequired) {
        if(!Auth.isLogged()){
          event.preventDefault();
          $rootScope.postLogInState = toState;
          $state.go('app.login');
        }
      }
    });

    /*
    * @ngdoc function
    * @name app.function:ordenarArraysObj
    * @description Funció per ordenar els elements d'un array d'objectes en funció del paràmetre passat
    * @function
    * @params {object} parametres Conté els camp pel que cerquem i l'ordre.
    * @public
    */
    $rootScope.ordenarArraysObj = function(parametres) {
        return function(a, b) {
          var value = 0;
          switch(parametres.ordre){
            case 'asc':
              if (a[parametres.camp] < b[parametres.camp])
                value = -1;
              else if (a[parametres.camp] > b[parametres.camp])
                value = 1;
              else
                value = 0;
              break;
            case 'desc':
              if (a[parametres.camp] < b[parametres.camp]){
                value = 1;
              } else if (a[parametres.camp] > b[parametres.camp]){
                value = -1;
              } else{
                value = 0;
              }
              break;
          }
          return value;
        }
      }
  });
});

//Constants

/**
 * @ngdoc object
 * @name app.URL_FIREBASE
 * @description URL per connectar amb la base de dades de Firebase
 * @example
 * URL_FIREBASE és una constant injectable (com un servei)
 * <pre>
 * (...)    .config(function (URL_FIREBASE) {
 * </pre>
 */
app.constant('URL_FIREBASE', 'https//jroig-tfc.firebaseio.com');
/**
 * @ngdoc object
 * @name app.APP_NAME
 * @description Constant que conté el nom de l'aplicació
 * @example
 * APP_NAME és una constant injectable (com un servei)
 * <pre>
 * (...)    .config(function (APP_NAME) {
 * </pre>
 */
app.constant('APP_NAME', 'comAPPcasa');


/*
------------------------------------
Filtres
------------------------------------
/**
 * @ngdoc filter
 * @name app.filter:rangs
 * @description
 * Filtre per autocompletar alguns selects de forma automàtica
 * utilitzant un màxim, mínim, i increment
 * @function
 * TODOparam {type=Object} input
 * @param {float} min Valor mínim del rang
 * @param {float} max Valor màxim del rang
 * @param {float} increment Increment entre cada valor
 */
app.filter('rangs', function() {
  return function(input, min, max, increment) {
    for (var i=min; i<=max; i += increment)
      input.push(i);
    return input;
  };
});